@@ -9,6 +9,7 @@ from _HTSeq import *
 
 from _version import __version__
 
+#from vcf_reader import *
 
 #########################
 ## Utils
@@ -299,7 +300,7 @@ class FastaReader( FileOrSequence ):
          else: 
             assert seq is not None, "FASTA file does not start with '>'."
             seq += line[:-1]
-      if seq:
+      if seq is not None:
          s = Sequence( seq, name )
          s.descr = descr
          yield s
@@ -320,6 +321,46 @@ class FastaReader( FileOrSequence ):
       if seqname is not None:
          seqlengths[ seqname ] = length
       return seqlengths
+      
+   @staticmethod   
+   def _import_pysam():
+      global pysam
+      try:
+         import pysam
+      except ImportError:
+         sys.stderr.write( "Please install the 'pysam' package to be able to use the Fasta indexing functionality." )
+         raise
+      
+   def build_index( self, force = False ):
+      self._import_pysam()
+      if not isinstance( self.fos, str ):
+         raise TypeError, "This function only works with FastaReader objects " + \
+            "connected to a fasta file via file name"
+      index_filename = self.fos + ".fai"
+      if os.access( index_filename, os.R_OK ):
+         if (not force) and os.stat( self.filename_or_sequence ).st_mtime <= \
+               os.stat( index_filename ).st_mtime:
+            # index is up to date
+            return
+      pysam.faidx( self.fos )
+      if not os.access( index_filename, os.R_OK ):
+         raise SystemError, "Building of Fasta index failed due to unknown error."
+      
+   def __getitem__( self, iv ):
+      if not isinstance( iv, GenomicInterval ):
+         raise TypeError, "GenomicInterval expected as key."
+      if not isinstance( self.fos, str ):
+         raise TypeError, "This function only works with FastaReader objects " + \
+            "connected to a fasta file via file name"
+      self._import_pysam()
+      fasta = pysam.faidx( self.fos, "%s:%d-%d" % ( iv.chrom, iv.start, iv.end-1 ) )
+      ans = list( FastaReader( fasta ) )
+      assert len( ans ) == 1
+      ans[0].name = str(iv)
+      if iv.strand != "-":
+         return ans[0]
+      else:
+         return ans[0].get_reverse_complement()
             
 class FastqReader( FileOrSequence ):
    """A Fastq object is associated with a FASTQ self.file. When an iterator
@@ -493,36 +534,18 @@ class SAM_Reader( FileOrSequence ):
    contains short read alignments. It can generate an iterator of Alignment
    objects."""
 
-   def __init__( self, filename_or_sequence ):      
-      FileOrSequence.__init__( self, filename_or_sequence )
-      self.peeked = None      
-
    def __iter__( self ):
-      if self.peeked is not None and self.peeked != []:
-         for read in self.peeked:
-            yield read
-         self.peeked = None
       for line in FileOrSequence.__iter__( self ):
          if line.startswith( "@" ):
             # do something with the header line
             continue
          try:
-            algnt = SAM_Alignment( line )
+            algnt = SAM_Alignment.from_SAM_line( line )
          except ValueError, e:
             e.args = e.args + ( self.get_line_number_string(), )
             raise
          yield algnt
        
-   def peek( self, num = 1 ):
-      i = 0
-      self.peeked = []
-      for read in self:
-         self.peeked.append( read )
-         i += 1
-         if i >= num:
-            break
-      return self.peeked
-
 class GenomicArrayOfSets( GenomicArray ):
 
    """A GenomicArrayOfSets is a specialization of GenomicArray that allows to store
@@ -532,23 +555,14 @@ class GenomicArrayOfSets( GenomicArray ):
    the present set, and the set is split if necessary.
    """
 
-   def __init__( self, chroms, stranded=True ):
-      GenomicArray.__init__( self, chroms, stranded, 'O' )
+   def __init__( self, chroms, stranded=True, storage='step', memmap_dir = "" ):
+      GenomicArray.__init__( self, chroms, stranded, 'O', storage, memmap_dir )
 
    def add_chrom( self, chrom, length = sys.maxint, start_index = 0 ):
       GenomicArray.add_chrom( self, chrom, length, start_index )
-      if self.stranded:
-         self.step_vectors[ chrom ][ "+" ][ : ] = set()
-         self.step_vectors[ chrom ][ "-" ][ : ] = set()
-      else:
-         self.step_vectors[ chrom ][ : ] = set()
-      
-   def add_value( self, value, iv ):
-      def _f( oldset ):
-         newset = oldset.copy()
-         newset.add( value )
-         return newset 
-      self.apply( _f, iv )
+      for cv in self.chrom_vectors[ chrom ].values():
+         cv[:] = set()
+         cv.is_vector_of_sets = True
       
        
 ###########################
@@ -602,3 +616,150 @@ def pair_SAM_alignments( alignments ):
    for p in process_list( almnt_list ):
       yield p
 
+
+###########################
+##   variant calls
+###########################
+
+
+_re_vcf_meta_comment = re.compile( "^##([a-zA-Z]+)\=(.*)$" )
+
+_re_vcf_meta_descr = re.compile('ID=[^,]+,?|Number=[^,]+,?|Type=[^,]+,?|Description="[^"]+",?')
+
+_re_vcf_meta_types = re.compile( "[INFO|FILTER|FORMAT]" )
+
+_vcf_typemap = {
+    "Integer":int,
+    "Float":float,
+    "String":str,
+    "Flag":bool
+}
+
+class VariantCall( object ):
+    
+    def __init__( self, line, nsamples = 0, sampleids=[] ):
+        if nsamples == 0:
+            self.format = None
+            self.chrom, self.pos, self.id, self.ref, self.alt, self.qual, self.filter, self.info = line.rstrip("\n").split("\t", 7)
+        else:
+            lsplit = line.rstrip("\n").split("\t")
+            self.chrom, self.pos, self.id, self.ref, self.alt, self.qual, self.filter, self.info = lsplit[:8]
+            self.format = lsplit[8].split(":")
+            self.samples = {}
+            spos=9
+            for sid in sampleids:
+                self.samples[ sid ] = dict( ( name, value ) for (name, value) in itertools.izip( self.format, lsplit[spos].split(":") ) )
+                spos += 1
+        self.pos = GenomicPosition( self.chrom, int(self.pos) )
+        self.alt = self.alt.split(",")
+    
+    def __descr__( self ):
+        return "<VariantCall at %s, ref '%s', alt %s >" % (str(self.pos).rstrip("/."), self.ref, str(self.alt).strip("[]"))
+    
+    def __str__( self ):
+        return "%s:'%s'->%s" % (str(self.pos).rstrip("/."), self.ref, str(self.alt).strip("[]"))
+    
+    def unpack_info( self, infodict ):
+        tmp = {}
+        for token in self.info.strip(";").split(";"):
+            if re.compile("=").search(token):
+                token = token.split("=")
+                if infodict.has_key( token[0] ):
+                    tmp[token[0]] = map( infodict[token[0]], token[1].split(",") )
+                else:
+                    tmp[token[0]] = token[1].split(",")
+                if len( tmp[ token[0] ] ) == 1:
+                    tmp[token[0]] = tmp[token[0]][0]
+            else: #Flag attribute found
+                tmp[token] = True
+        diff = set( infodict.keys() ).difference( set( tmp.keys() ) )
+        for key in diff:
+            if infodict[key] == bool:
+                tmp[key] = False
+        self.info = tmp
+
+class VCF_Reader( FileOrSequence ):
+
+    def __init__( self, filename_or_sequence ):
+        FileOrSequence.__init__( self, filename_or_sequence )
+        self.metadata = {}
+        self.info = {}
+        self.filters = {}
+        self.formats = {}
+        self.nsamples = 0
+        self.sampleids = []
+        
+    def make_info_dict( self ):
+        self.infodict = dict( ( key, _vcf_typemap[self.info[key]["Type"]] ) for key in self.info.keys() )
+    
+    def parse_meta( self, header_filename = None ):
+        if header_filename == None:
+            the_iter = FileOrSequence.__iter__( self )
+        else:
+            the_iter = open( header_filename, "r" )
+        
+        for line in the_iter:
+            if line.startswith( '#' ):
+                if line.startswith( "##" ):
+                    mo = _re_vcf_meta_comment.match( line )
+                    if mo:
+                        value = mo.group(2)
+                        if mo.group(1) == "INFO":
+                            value = dict( e.rstrip(",").split("=",1) for e in _re_vcf_meta_descr.findall(value) )
+                            key = value["ID"]
+                            del value["ID"]
+                            self.info[ key ] = value
+                        elif mo.group(1) == "FILTER":
+                            value = dict( e.rstrip(",").split("=",1) for e in _re_vcf_meta_descr.findall(value) )
+                            key = value["ID"]
+                            del value["ID"]
+                            self.filters[ key ] = value
+                        elif mo.group(1) == "FORMAT":
+                            value = dict( e.rstrip(",").split("=",1) for e in _re_vcf_meta_descr.findall(value) )
+                            key = value["ID"]
+                            del value["ID"]
+                            self.formats[ key ] = value
+                        else:
+                            self.metadata[ mo.group(1) ] = mo.group(2)
+                else:
+                    self.sampleids = line.rstrip("\t\n").split("\t")[9:]
+                    self.nsamples = len( self.sampleids )
+                continue
+            else:
+                break
+    
+    def __iter__( self ):
+        for line in FileOrSequence.__iter__( self ):
+            if line == "\n" or line.startswith( '#' ):
+                continue
+            vc = VariantCall( line, self.nsamples, self.sampleids )
+            yield vc
+
+class BAM_Reader( object ):
+
+    def __init__( self, filename ):
+        global pysam
+        self.filename = filename
+        self.sf = None  # This one is only used by __getitem__
+        try:
+           import pysam
+        except ImportError:
+           print "Please Install PySam to use the BAM_Reader Class (http://code.google.com/p/pysam/)"
+           raise
+    
+    def __iter__( self ):
+        sf = pysam.Samfile(self.filename, "rb")
+        for pa in sf:
+            yield SAM_Alignment.from_pysam_AlignedRead( pa, sf )
+
+    def __getitem__( self, iv ):
+        if not isinstance( iv, GenomicInterval ):
+           raise TypeError, "Use a HTSeq.GenomicInterval to access regions within .bam-file!"        
+        if self.sf is None:
+           self.sf = pysam.Samfile( self.filename, "rb" )
+           if not self.sf._hasIndex():
+              raise ValueError, "The .bam-file has no index, random-access is disabled!"
+        for pa in self.sf.fetch( iv.chrom, iv.start+1, iv.end ):
+            yield SAM_Alignment.from_pysam_AlignedRead( pa, self.sf )
+    
+               
